// Author 		: F. de la Parra
// Date			: February, 2019
// Description 	: JavaDemo5 exercises wrapper classes, Math class,
//				  String class, StringTokenizer class, and method
//				  overloading. 
// Version		: 1.0

import java.util.Scanner;
import java.util.StringTokenizer;
import java.util.InputMismatchException;
import static java.lang.Math.pow;

public class JavaDemo5 {
	
	// Method "main" drives topic selection
	
	public static void main(String[] args) {
		
		// Create a 'demo' object.
		JavaDemo5 demoObject = new JavaDemo5();
		
		// Create Scanner object 'menuSelect' for input from keyboard. 
		Scanner menuSelect = new Scanner(System.in);
		
		// Initialize menuItem selection.
		int menuItem = 0;
		
		// Display menu and wait for a menu item selection
		while (menuItem != 9) {
			// Display menu
			System.out.println("\n***JavaDemo5 Menu ***");
			System.out.println("1 - Wrapper Classes");
			System.out.println("2 - Math Class");
			System.out.println("3 - String");
			System.out.println("4 - StringTokenizer");
			System.out.println("5 - Method overloading");
			System.out.println("9 - Exit");
			System.out.println();
			System.out.println("Enter demo number:");
			
			// Input a menu selection.
			menuItem = menuSelect.nextInt();
			
			// Run demo corresponding to selected menu item.
			switch (menuItem) {
				
				case 1 : {
					System.out.println("\n*** Running wrapper classes demo ***\n");
					
					// Execute wrapper classes demo.
					demoObject.wrapperClassesDemo();
					break;
				}
				
				case 2 : {
					System.out.println("\n*** Running Math class demo ***\n");
					
					// Execute Math class demo.
					demoObject.mathClassDemo();
					break;
				}
				
				case 3 : {
					System.out.println("\n*** Running String class demo ***\n");
					
					// Execute String class demo.
					demoObject.stringClassDemo();
					break;
				}
				
				case 4 : {
					System.out.println("\n*** Running StringTokenizer class demo ***\n");
					
					// Execute StringTokenizer class demo.
					demoObject.stringTokenizerClassDemo();
					break;
				}
				
				case 5 : {
					System.out.println("\n*** Running method overload demo ***\n");
					
					// Execute method overloading demo.
					demoObject.overloadDemo();
					break;
				}
				
			 // case n :     More upcoming demos ...
				
				default : {
					System.out.println("Either 9 or invalid selection entered");
					break;
				}
			}
			System.out.println("\n*** Demo completed ***\n");
		}
		
		System.out.println("\n*** Program terminated ***\n");
	}
	
	// Method wrapperClassesDemo shows wrapper classes in action.

	public void wrapperClassesDemo() {
	
		// Wrapper classes include Byte, Short, Long, Float, Double, and Character, for the
		// primitives byte, short, long, float, double and char, respectively.
		
		// Creating a new Integer object using constructor.
		// Integer integerObject = new Integer(25); (*** Deprecated ***)
		
		// Automatic BOXING.
		Integer integerObject = 25;
		System.out.println("\nintegerObject is " + integerObject);
		
		// Instance methods
		String integerString = integerObject.toString();
		System.out.print("Displaying integer " + integerString + " as a string: ");
		System.out.println("\"" + integerString + "\"");
		
		// Integer class UNBOXING
		// Using an instance method.
		int primitiveInt = integerObject.intValue();
		System.out.println("Variable primitiveInt is " + primitiveInt);
		
		// Using automatic UNBOXING
		int primitiveInt2 = integerObject;
		System.out.println("Variable primitiveInt2 is " + primitiveInt2);
		
		// Integer class static methods. Parse a numeric string an covert it to an integer.
		int primitiveInt3 = Integer.parseInt("123");
		System.out.println("Variable primitiveInt3 is " + primitiveInt3);
		
		// Display the binary representation of variable primitiveInt3.
		System.out.println("Variable primitiveInt3 in binary is " + Integer.toBinaryString(primitiveInt3));

		// Character class static methods.
		System.out.println("\nApplying static methods");
		System.out.println("'a' in uppercase is " + Character.toUpperCase('a'));
		System.out.println("'B' is a lower case character. It is " + Character.isLowerCase('B'));
		System.out.println("'5' is a digit. It is " + Character.isDigit('5'));
		System.out.println("'d' is a letter. It is " + Character.isLetter('d'));
		System.out.println("65 is the unicode for " + Character.getName(65));
		
		// Character instance methods.
		Character c = 'p';
		System.out.println("\nApplying instance methods:");
		System.out.println("The value of character 'p' is " + c.charValue());
		System.out.println("The value of character 'p' as string is " + c.toString());
	}
	
	// Method mathClassDemo exercises the methods in the Math class.
	
	public void mathClassDemo() {
		
		// The Math class implements a series of static methods to compute mathematical functions and
		// the constants PI and E.
		
		// Constants.
		System.out.println("\nCONSTANTS");
		System.out.println("The value of PI is " + Math.PI);
		System.out.println("The value of E is " + Math.E);
		
		// Functions
		System.out.println("\nFUNCTIONS");
		// The 'pow' call uses the import static reference.
		System.out.println("pow(E, PI) is " + pow(Math.E, Math.PI));
		
		// The remaining functions need the class name (no existing "import static").
		System.out.println("cos(PI) is " + Math.cos(Math.PI));
		System.out.println("ceil(PI) is " + Math.ceil(Math.PI));
		System.out.println("floor(E) is " + Math.floor(Math.E));
		System.out.println("round(E) is " + Math.round(Math.E));
		System.out.println("abs(-1) is " + Math.abs(-1));
		
	}
	
	// Method stringClassDemo exercises methods in the String class.
	
	public void stringClassDemo() {
		
		// Static methods
		System.out.println("String class - STATIC METHODS");
		char[] helloChars = {'H','e','l','l','o'};
		String stringObject = String.valueOf(helloChars);
		System.out.println("stringObject's value is " + stringObject);
		
		// Instance methods
		System.out.println("\nString class - INSTANCE METHODS");
		String phrase = new String("Rise to vote, Sir.");
		System.out.println("Selected phrase is : " + phrase);
		System.out.println("In upper case is : " + phrase.toUpperCase());
		System.out.println("In lower case is : " + phrase.toLowerCase());
		System.out.println("It contains " + phrase.length() + " characters");
		System.out.println("Substring \"vote\" is in position " + phrase.indexOf("vote"));
		System.out.println("No punctuation results in : " + phrase.replaceAll(",", ""));
	}
	
	// Method stringTokenizerClassDemo displays a list of tokens.
	
	public void stringTokenizerClassDemo() {
		
		// stringTokenizer demo.
		
		// Build a phrase without commas
		String phrase = "Rise to vote, Sir.";
		String phrase1 = phrase.replaceAll(",", "");
		System.out.println("phrase without commas is : " + phrase1);
		
		// Tokenize the phrase.
		StringTokenizer wordList = new StringTokenizer(phrase1);
		
		// Display the tokens.
		while (wordList.hasMoreTokens())
			System.out.println(wordList.nextToken());
	}
	
	public void overloadDemo() {
		String stringVar = "Charles";
		char[] charVar = {'C', 'h', 'a', 'r', 'l', 'e', 's'};
		
		System.out.println("String \"Charles\" reversed is " + revString(stringVar));
		System.out.println("\nCharacter array Charles reversed is " + revString(charVar));
	};
	
	// AUXILIARY (private) METHODS.

	// Method revString takes a 'string' of characters and 
	// generates it backwards.
	
	private String revString(String in) {
		char[] inChars = in.toCharArray();
		char[] outChars = new char[in.length()];
		
		for (int i = 0; i < in.length(); i++)
			outChars[i] = inChars[in.length() - i - 1];
		
		return String.valueOf(outChars);
	}
	
	// Method revString takes an array of characters and
	// generates a string spelled backwards.
	
	private String revString(char[] in) {
		char[] outChars = new char[in.length];
		
		for (int i = 0; i < in.length; i++)
			outChars[i] = in[in.length - i - 1];
		
		return String.valueOf(outChars);
	}
}